package com.zanxiang.game.module.manage.websocket;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.zanxiang.erp.base.pojo.TokenInfo;
import com.zanxiang.erp.security.util.SecurityUtil;
import com.zanxiang.game.module.manage.constant.RedisKeyConstant;
import com.zanxiang.game.module.manage.enums.KfWebSocketMsgEnum;
import com.zanxiang.game.module.manage.pojo.dto.KfWebSocketMsgDTO;
import com.zanxiang.game.module.manage.pojo.params.KfWebSocketMsgParam;
import com.zanxiang.game.module.manage.service.IKfRoomService;
import com.zanxiang.game.module.manage.service.IKfSessionUserService;
import com.zanxiang.game.module.mybatis.entity.KfRoom;
import com.zanxiang.game.module.mybatis.entity.KfSessionUser;
import com.zanxiang.module.util.JsonUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.web.socket.CloseStatus;
import org.springframework.web.socket.WebSocketHandler;
import org.springframework.web.socket.WebSocketMessage;
import org.springframework.web.socket.WebSocketSession;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;

/**
 * @author : lingfeng
 * @time : 2024-02-23
 * @description : 客服消息Websocket处理器
 */
@Slf4j
@Lazy(value = false)
@Component
public class KfMsgWebsocketHandler implements WebSocketHandler {

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Autowired
    private KfMsgWebSocketSessionRegistry kfMsgWebSocketSessionRegistry;

    @Autowired
    private IKfRoomService kfRoomService;

    @Autowired
    private IKfSessionUserService kfSessionUserService;

    /**
     * websocket连接建立成功
     */
    @Override
    public void afterConnectionEstablished(WebSocketSession session) throws Exception {
    }

    /**
     * 收到消息
     */
    @Override
    public void handleMessage(WebSocketSession session, WebSocketMessage<?> message) throws Exception {
        String msgStr = message.getPayload().toString();
        log.error("收到前端消息 msgStr : {}", msgStr);
        //消息解析
        KfWebSocketMsgParam kfWebSocketMsgParam = JsonUtil.toObj(msgStr, KfWebSocketMsgParam.class);
        log.error("收到前端消息解析结果 kfWebSocketMsgParam : {}", JsonUtil.toString(kfWebSocketMsgParam));
        //消息类型
        KfWebSocketMsgEnum webSocketMsgType = kfWebSocketMsgParam.getWebSocketMsgType();
        //请求令牌
        String token = kfWebSocketMsgParam.getToken();
        //连接身份验证
        boolean tokenCheckResult = this.tokenCheck(webSocketMsgType, token);
        //身份验证失败, 没有后续动作
        if (!tokenCheckResult) {
            log.error("token验证失败, token : {}", token);
            return;
        }
        //握手-消息处理
        if (Objects.equals(kfWebSocketMsgParam.getWebSocketMsgType(), KfWebSocketMsgEnum.WEBSOCKET_MSG_KF_HAND_SHAKE)) {
            log.error("首次握手, kfUserId : {}", SecurityUtil.getUserId());
            this.kfHandShake(session, webSocketMsgType);
        }
        //创建连接-消息处理
        if (Objects.equals(kfWebSocketMsgParam.getWebSocketMsgType(), KfWebSocketMsgEnum.WEBSOCKET_MSG_KF_CREATE_CONNECT)) {
            log.error("创建连接, kfUserId : {}", SecurityUtil.getUserId());
            this.kfCreateConnect(webSocketMsgType);
        }
        //玩家接入-消息处理
        if (Objects.equals(kfWebSocketMsgParam.getWebSocketMsgType(), KfWebSocketMsgEnum.WEBSOCKET_MSG_USER_CONNECT_JOIN)) {
            log.error("玩家接入, kfUserId : {}", SecurityUtil.getUserId());
            this.userConnectJoin(kfWebSocketMsgParam);
        }
        //获取房间历史消息-消息处理
        if (Objects.equals(kfWebSocketMsgParam.getWebSocketMsgType(), KfWebSocketMsgEnum.WEBSOCKET_MSG_ROOM_HISTORY)) {
            log.error("获取房间历史消息, kfUserId : {}", SecurityUtil.getUserId());
        }
        //客服发送消息-消息处理
        if (Objects.equals(kfWebSocketMsgParam.getWebSocketMsgType(), KfWebSocketMsgEnum.WEBSOCKET_MSG_KF_SEND)) {

        }
        //获取已结束房间列表-消息处理
        if (Objects.equals(kfWebSocketMsgParam.getWebSocketMsgType(), KfWebSocketMsgEnum.WEBSOCKET_MSG_FINISH_ROOM_LIST)) {

        }
        //结束会话-消息处理
        if (Objects.equals(kfWebSocketMsgParam.getWebSocketMsgType(), KfWebSocketMsgEnum.WEBSOCKET_MSG_FINISH_SESSION)) {

        }
        //快捷回复-消息处理
        if (Objects.equals(kfWebSocketMsgParam.getWebSocketMsgType(), KfWebSocketMsgEnum.WEBSOCKET_MSG_QUICK_REPLY)) {

        }
    }

    /**
     * 消息传输发生错误
     */
    @Override
    public void handleTransportError(WebSocketSession session, Throwable exception) throws Exception {
    }

    /**
     * 连接被关闭
     */
    @Override
    public void afterConnectionClosed(WebSocketSession session, CloseStatus closeStatus) throws Exception {
        //从session中获取客服id
        Long kfUserId = Long.valueOf(session.getAttributes().get("kfUserId").toString());
        //关闭session关联的房间, 有在线状态的更新为断开状态
        kfRoomService.update(new LambdaUpdateWrapper<KfRoom>()
                .set(KfRoom::getOnline, Boolean.FALSE)
                .set(KfRoom::getUpdateTime, LocalDateTime.now())
                .eq(KfRoom::getKfUserId, kfUserId));
        //关闭连接
        session.close();
        //移除连接
        kfMsgWebSocketSessionRegistry.removeSession(kfUserId);
    }

    @Override
    public boolean supportsPartialMessages() {
        return false;
    }

    private void msgRoomHistory(KfWebSocketMsgParam param) {
        if (param.getRoomId() == null || param.getPage() == null) {
            this.pushMessage(KfWebSocketMsgDTO.fail(param.getWebSocketMsgType(),
                    "获取房间历史消息参数错误, roomId和page都不可为空, param : " + JsonUtil.toString(param)));
        }
    }

    private void userConnectJoin(KfWebSocketMsgParam param) {
        if (Strings.isBlank(param.getOpenId()) || param.getGameId() == null) {
            this.pushMessage(KfWebSocketMsgDTO.fail(param.getWebSocketMsgType(),
                    "接入玩家参数错误, openId和gameId都不可为空, param : " + JsonUtil.toString(param)));
        }
        //玩家更新
        kfSessionUserService.update(new LambdaUpdateWrapper<KfSessionUser>()
                .set(KfSessionUser::getIsWait, Boolean.FALSE)
                .set(KfSessionUser::getUpdateTime, LocalDateTime.now()));
        //房间更新
        kfRoomService.userJoinRoom(param.getOpenId(), param.getGameId());
        //发送消息, 给该客服返回完整的已接入房间列表
        List<KfWebSocketMsgDTO.RoomBean> roomList = kfRoomService.getOnlineRoomList();
        this.pushMessage(KfWebSocketMsgDTO.builder()
                .webSocketMsgType(param.getWebSocketMsgType())
                .kfUserId(SecurityUtil.getUserId())
                .roomList(roomList)
                .build());
        //发送消息, 给所有在线客服推送完整待接入列表
        List<KfWebSocketMsgDTO.WaitUserBean> waitUserList = kfSessionUserService.getWaitUserList();
        this.pushMessage(KfWebSocketMsgDTO.builder()
                .webSocketMsgType(param.getWebSocketMsgType())
                .waitUserList(waitUserList)
                .build());
    }

    private void kfCreateConnect(KfWebSocketMsgEnum msgTypeEnum) {
        //获取待接入列表
        List<KfWebSocketMsgDTO.WaitUserBean> waitUserList = kfSessionUserService.getWaitUserList();
        //获取已接入房间列表
        List<KfWebSocketMsgDTO.RoomBean> roomList = kfRoomService.getOnlineRoomList();
        //发送消息
        this.pushMessage(KfWebSocketMsgDTO.builder()
                .webSocketMsgType(msgTypeEnum)
                .kfUserId(SecurityUtil.getUserId())
                .waitUserList(waitUserList)
                .roomList(roomList)
                .build());
    }

    private void kfHandShake(WebSocketSession session, KfWebSocketMsgEnum msgTypeEnum) {
        //添加会话
        Long kfUserId = SecurityUtil.getUserId();
        session.getAttributes().put("kfUserId", kfUserId);
        kfMsgWebSocketSessionRegistry.addSession(kfUserId, session);
        //获取游戏列表
        List<KfWebSocketMsgDTO.GameBean> gameList = kfRoomService.getKfGameList();
        //发送消息
        this.pushMessage(KfWebSocketMsgDTO.builder()
                .webSocketMsgType(msgTypeEnum)
                .kfUserId(SecurityUtil.getUserId())
                .gameList(gameList)
                .build());
    }

    private boolean tokenCheck(KfWebSocketMsgEnum webSocketMsgType, String token) {
        //令牌为空
        if (Strings.isBlank(token)) {
            this.pushMessage(KfWebSocketMsgDTO.fail(webSocketMsgType, "非法参数, token令牌不可为空"));
            return Boolean.FALSE;
        }
        //令牌验证
        TokenInfo tokenInfo = SecurityUtil.parseToken(token);
        if (tokenInfo == null) {
            this.pushMessage(KfWebSocketMsgDTO.fail(webSocketMsgType, "参数错误, 令牌验证不通过"));
            return Boolean.FALSE;
        }
        //将token设置到当前线程
        SecurityUtil.fillToken(token);
        //返回验证通过
        return Boolean.TRUE;
    }

    /**
     * 消息发送到redis广播
     */
    public void pushMessage(KfWebSocketMsgDTO kfWebSocketMsgDTO) {
        redisTemplate.convertAndSend(RedisKeyConstant.KF_MSG_REDIS_LISTEN_TOPIC, JsonUtil.toString(kfWebSocketMsgDTO));
    }
}
